@@ -1,4 +1,26 @@
-#!/usr/bin/env python
+# coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 
 """
 Reads the header from a GE a-Si Angio Detector
@@ -21,9 +43,9 @@ Authors: Henning O. Sorensen & Erik Knudsen
 # Get ready for python3:
 from __future__ import with_statement, print_function
 import numpy
-from .fabioimage import fabioimage
+from .fabioimage import FabioImage
 
-class GEimage(fabioimage):
+class GEimage(FabioImage):
 
 
     def _readheader(self, infile):
@@ -462,7 +484,7 @@ class GEimage(fabioimage):
         Read in header into self.header and
             the data   into self.data
         """
-        self.header = {}
+        self.header = self.check_header()
         self.resetvals()
         infile = self._open(fname, "rb")
         self._readheader(infile)
@@ -498,3 +520,5 @@ class GEimage(fabioimage):
         self.bytecode = self.data.dtype.type
         self.pilimage = None
         return self
+
+GEimage = GeImage
