@@ -1,5 +1,27 @@
-#!/usr/bin/env python
-#coding: utf8
+# coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 """
 Reads Oxford Diffraction Sapphire 3 images
 
@@ -23,12 +45,13 @@ from __future__ import with_statement, print_function
 import time, logging, struct
 logger = logging.getLogger("OXDimage")
 import numpy
-from .fabioimage import fabioimage
+from .fabioimage import FabioImage
 from .compression import decTY1, compTY1
+from .fabioutils import to_str
 
 try:
     from numpy import rad2deg, deg2rad
-except ImportError: #naive implementation for very old numpy (v1.0.1 on MacOSX from Risoe)
+except ImportError:  # naive implementation for very old numpy (v1.0.1 on MacOSX from Risoe)
     rad2deg = lambda x: 180.0 * x / numpy.pi
     deg2rad = lambda x: x * numpy.pi / 180.
 
@@ -50,18 +73,20 @@ DEFAULT_HEADERS = {'Header Version':  'OD SAPPHIRE  3.0',
                    'NSUPPLEMENT':0
                    }
 
-class OXDimage(fabioimage):
+class OxdImage(FabioImage):
     """
     Oxford Diffraction Sapphire 3 images reader/writer class
+
+    Note: We assume the binary format is alway little-endian, is this True ?
     """
     def _readheader(self, infile):
 
         infile.seek(0)
 
         # Ascii header part 256 byes long
-        self.header['Header Version'] = infile.readline()[:-2]
+        self.header['Header Version'] = to_str(infile.readline()[:-2])
         block = infile.readline()
-        self.header['Compression'] = block[12:15]
+        self.header['Compression'] = to_str(block[12:15])
         block = infile.readline()
         self.header['NX'] = int(block[3:7])
         self.header['NY'] = int(block[11:15])
@@ -77,7 +102,7 @@ class OXDimage(fabioimage):
         block = infile.readline()
         self.header['NSUPPLEMENT'] = int(block[12:19])
         block = infile.readline()
-        self.header['Time'] = block[5:29]
+        self.header['Time'] = to_str(block[5:29])
         self.header["ASCII Section size in Byte"] = self.header['Header Size In Bytes']\
                                                 - self.header['General Section size in Byte']\
                                                 - self.header['Special Section size in Byte'] \
@@ -87,41 +112,47 @@ class OXDimage(fabioimage):
         # Skip to general section (NG) 512 byes long <<<<<<"
         infile.seek(self.header["ASCII Section size in Byte"])
         block = infile.read(self.header['General Section size in Byte'])
-        self.header['Binning in x'] = numpy.fromstring(block[0:2], numpy.uint16)[0]
-        self.header['Binning in y'] = numpy.fromstring(block[2:4], numpy.uint16)[0]
-        self.header['Detector size x'] = numpy.fromstring(block[22:24], numpy.uint16)[0]
-        self.header['Detector size y'] = numpy.fromstring(block[24:26], numpy.uint16)[0]
-        self.header['Pixels in x'] = numpy.fromstring(block[26:28], numpy.uint16)[0]
-        self.header['Pixels in y'] = numpy.fromstring(block[28:30], numpy.uint16)[0]
-        self.header['No of pixels'] = numpy.fromstring(block[36:40], numpy.uint32)[0]
+        self.header['Binning in x'] = struct.unpack("<H", block[0:2])[0]
+        self.header['Binning in y'] = struct.unpack("<H", block[2:4])[0]
+        self.header['Detector size x'] = struct.unpack("<H", block[22:24])[0]
+        self.header['Detector size y'] = struct.unpack("<H", block[24:26])[0]
+        self.header['Pixels in x'] = struct.unpack("<H", block[26:28])[0]
+        self.header['Pixels in y'] = struct.unpack("<H", block[28:30])[0]
+        self.header['No of pixels'] = struct.unpack("<I", block[36:40])[0]
 
         # Speciel section (NS) 768 bytes long
         block = infile.read(self.header['Special Section size in Byte'])
-        self.header['Gain'] = numpy.fromstring(block[56:64], numpy.float)[0]
-        self.header['Overflows flag'] = numpy.fromstring(block[464:466], numpy.int16)[0]
-        self.header['Overflow after remeasure flag'] = numpy.fromstring(block[466:468], numpy.int16)[0]
-        self.header['Overflow threshold'] = numpy.fromstring(block[472:476], numpy.int32)[0]
-        self.header['Exposure time in sec'] = numpy.fromstring(block[480:488], numpy.float)[0]
-        self.header['Overflow time in sec'] = numpy.fromstring(block[488:496], numpy.float)[0]
-        self.header['Monitor counts of raw image 1'] = numpy.fromstring(block[528:532], numpy.int32)[0]
-        self.header['Monitor counts of raw image 2'] = numpy.fromstring(block[532:536], numpy.int32)[0]
-        self.header['Monitor counts of overflow raw image 1'] = numpy.fromstring(block[536:540], numpy.int32)[0]
-        self.header['Monitor counts of overflow raw image 2'] = numpy.fromstring(block[540:544], numpy.int32)[0]
-        self.header['Unwarping'] = numpy.fromstring(block[544:548], numpy.int32)[0]
-        self.header['Detector type'] = DETECTOR_TYPES[numpy.fromstring(block[548:552], numpy.int32)[0]]
-        self.header['Real pixel size x (mm)'] = numpy.fromstring(block[568:576], numpy.float)[0]
-        self.header['Real pixel size y (mm)'] = numpy.fromstring(block[576:584], numpy.float)[0]
+        self.header['Gain'] = struct.unpack("<d", block[56:64])[0]
+        self.header['Overflows flag'] = struct.unpack("<h", block[464:466])[0]
+        self.header['Overflow after remeasure flag'] = struct.unpack("<h", block[466:468])[0]
+        self.header['Overflow threshold'] = struct.unpack("<i", block[472:476])[0]
+        self.header['Exposure time in sec'] = struct.unpack("<d", block[480:488])[0]
+        self.header['Overflow time in sec'] = struct.unpack("<d", block[488:496])[0]
+        self.header['Monitor counts of raw image 1'] = struct.unpack("<i", block[528:532])[0]
+        self.header['Monitor counts of raw image 2'] = struct.unpack("<i", block[532:536])[0]
+        self.header['Monitor counts of overflow raw image 1'] = struct.unpack("<i", block[536:540])[0]
+        self.header['Monitor counts of overflow raw image 2'] = struct.unpack("<i", block[540:544])[0]
+        self.header['Unwarping'] = struct.unpack("<i", block[544:548])[0]
+        self.header['Detector type'] = DETECTOR_TYPES[struct.unpack("<i", block[548:552])[0]]
+        self.header['Real pixel size x (mm)'] = struct.unpack("<d", block[568:576])[0]
+        self.header['Real pixel size y (mm)'] = struct.unpack("<d", block[576:584])[0]
 
         # KM4 goniometer section (NK) 1024 bytes long
         block = infile.read(self.header['KM4 Section size in Byte'])
         # Spatial correction file
-        self.header['Spatial correction file'] = block[26:272].strip("\x00")
-        self.header['Spatial correction file date'] = block[0:26].strip("\x00")
+        self.header['Spatial correction file'] = to_str(block[26:272].strip(b"\x00"))
+        self.header['Spatial correction file date'] = to_str(block[0:26].strip(b"\x00"))
         # Angles are in steps due to stepper motors - conversion factor RAD
-        # angle[0] = omega, angle[1] = theta, angle[2] = kappa, angle[3] = phi,   
+        # angle[0] = omega, angle[1] = theta, angle[2] = kappa, angle[3] = phi,
         start_angles_step = numpy.fromstring(block[284:304], numpy.int32)
         end_angles_step = numpy.fromstring(block[324:344], numpy.int32)
         step2rad = numpy.fromstring(block[368:408], numpy.float)
+        zero_correction_soft_step = numpy.fromstring(block[512:532], numpy.int32)
+        if not numpy.little_endian:
+            start_angles_step.byteswap(True)
+            end_angles_step.byteswap(True)
+            step2rad.byteswap(True)
+            zero_correction_soft_step.byteswap(True)
         step_angles_deg = rad2deg(step2rad)
         # calc angles
         start_angles_deg = start_angles_step * step_angles_deg
@@ -140,97 +171,113 @@ class OXDimage(fabioimage):
         self.header['Phi step in deg'] = step_angles_deg[3]
 
 
-        zero_correction_soft_step = numpy.fromstring(block[512:532], numpy.int32)
         zero_correction_soft_deg = zero_correction_soft_step * step_angles_deg
         self.header['Omega zero corr. in deg'] = zero_correction_soft_deg[0]
         self.header['Theta zero corr. in deg'] = zero_correction_soft_deg[1]
         self.header['Kappa zero corr. in deg'] = zero_correction_soft_deg[2]
         self.header['Phi zero corr. in deg'] = zero_correction_soft_deg[3]
         # Beam rotation about e2,e3
-        self.header['Beam rot in deg (e2)'] = numpy.fromstring(block[552:560], numpy.float)[0]
-        self.header['Beam rot in deg (e3)'] = numpy.fromstring(block[560:568], numpy.float)[0]
+        self.header['Beam rot in deg (e2)'] = struct.unpack("<d", block[552:560])[0]
+        self.header['Beam rot in deg (e3)'] = struct.unpack("<d", block[560:568])[0]
         # Wavelenghts alpha1, alpha2, beta
-        self.header['Wavelength alpha1'] = numpy.fromstring(block[568:576], numpy.float)[0]
-        self.header['Wavelength alpha2'] = numpy.fromstring(block[576:584], numpy.float)[0]
-        self.header['Wavelength alpha'] = numpy.fromstring(block[584:592], numpy.float)[0]
-        self.header['Wavelength beta'] = numpy.fromstring(block[592:600], numpy.float)[0]
+        self.header['Wavelength alpha1'] = struct.unpack("<d", block[568:576])[0]
+        self.header['Wavelength alpha2'] = struct.unpack("<d", block[576:584])[0]
+        self.header['Wavelength alpha'] = struct.unpack("<d", block[584:592])[0]
+        self.header['Wavelength beta'] = struct.unpack("<d", block[592:600])[0]
 
         # Detector tilts around e1,e2,e3 in deg
-        self.header['Detector tilt e1 in deg'] = numpy.fromstring(block[640:648], numpy.float)[0]
-        self.header['Detector tilt e2 in deg'] = numpy.fromstring(block[648:656], numpy.float)[0]
-        self.header['Detector tilt e3 in deg'] = numpy.fromstring(block[656:664], numpy.float)[0]
+        self.header['Detector tilt e1 in deg'] = struct.unpack("<d", block[640:648])[0]
+        self.header['Detector tilt e2 in deg'] = struct.unpack("<d", block[648:656])[0]
+        self.header['Detector tilt e3 in deg'] = struct.unpack("<d", block[656:664])[0]
 
 
         # Beam center
-        self.header['Beam center x'] = numpy.fromstring(block[664:672], numpy.float)[0]
-        self.header['Beam center y'] = numpy.fromstring(block[672:680], numpy.float)[0]
+        self.header['Beam center x'] = struct.unpack("<d", block[664:672])[0]
+        self.header['Beam center y'] = struct.unpack("<d", block[672:680])[0]
         # Angle (alpha) between kappa rotation axis and e3 (ideally 50 deg)
-        self.header['Alpha angle in deg'] = numpy.fromstring(block[672:680], numpy.float)[0]
+        self.header['Alpha angle in deg'] = struct.unpack("<d", block[672:680])[0]
         # Angle (beta) between phi rotation axis and e3 (ideally 0 deg)
-        self.header['Beta angle in deg'] = numpy.fromstring(block[672:680], numpy.float)[0]
+        self.header['Beta angle in deg'] = struct.unpack("<d", block[672:680])[0]
 
         # Detector distance
-        self.header['Distance in mm'] = numpy.fromstring(block[712:720], numpy.float)[0]
+        self.header['Distance in mm'] = struct.unpack("<d", block[712:720])[0]
         # Statistics section (NS) 512 bytes long
         block = infile.read(self.header['Statistic Section in Byte'])
-        self.header['Stat: Min '] = numpy.fromstring(block[0:4], numpy.int32)[0]
-        self.header['Stat: Max '] = numpy.fromstring(block[4:8], numpy.int32)[0]
-        self.header['Stat: Average '] = numpy.fromstring(block[24:32], numpy.float)[0]
-        self.header['Stat: Stddev '] = numpy.sqrt(numpy.fromstring(block[32:40], numpy.float)[0])
-        self.header['Stat: Skewness '] = numpy.fromstring(block[40:48], numpy.float)[0]
+        self.header['Stat: Min '] = struct.unpack("<i", block[0:4])[0]
+        self.header['Stat: Max '] = struct.unpack("<i", block[4:8])[0]
+        self.header['Stat: Average '] = struct.unpack("<d", block[24:32])[0]
+        self.header['Stat: Stddev '] = numpy.sqrt(struct.unpack("<d", block[32:40])[0])
+        self.header['Stat: Skewness '] = struct.unpack("<d", block[40:48])[0]
 
         # History section (NH) 2048 bytes long
         block = infile.read(self.header['History Section in Byte'])
-        self.header['Flood field image'] = block[99:126].strip("\x00")
+        self.header['Flood field image'] = to_str(block[99:126].strip(b"\x00"))
 
     def read(self, fname, frame=None):
         """
         Read in header into self.header and
             the data   into self.data
         """
-        self.header = {}
+        self.header = self.check_header()
         self.resetvals()
-        infile = self._open(fname)
-        self._readheader(infile)
+        with self._open(fname) as infile:
+            self._readheader(infile)
 
-        infile.seek(self.header['Header Size In Bytes'])
+            infile.seek(self.header['Header Size In Bytes'])
 
-        # Compute image size
-        try:
-            self.dim1 = int(self.header['NX'])
-            self.dim2 = int(self.header['NY'])
-        except:
-            raise Exception("Oxford  file", str(fname) + \
-                                "is corrupt, cannot read it")
-        #
-        if self.header['Compression'] == 'TY1':
-            #Compressed with the KM4CCD compression
-            raw8 = infile.read(self.dim1 * self.dim2)
-            raw16 = None
-            raw32 = None
-            if self.header['OI'] > 0:
-                raw16 = infile.read(self.header['OI'] * 2)
-            if self.header['OL'] > 0:
-                raw32 = infile.read(self.header['OL'] * 4)
-            #DEBUG stuff ... 
-            self.raw8 = raw8
-            self.raw16 = raw16
-            self.raw32 = raw32
-            #END DEBUG
-            block = decTY1(raw8, raw16, raw32)
-            bytecode = block.dtype
+            # Compute image size
+            try:
+                self.dim1 = int(self.header['NX'])
+                self.dim2 = int(self.header['NY'])
+            except:
+                raise Exception("Oxford  file", str(fname) + \
+                                    "is corrupt, cannot read it")
+            #
+            if self.header['Compression'] == 'TY1':
+                logger.debug("# Compressed with the KM4CCD compression")
+                raw8 = infile.read(self.dim1 * self.dim2)
+                raw16 = None
+                raw32 = None
+                if self.header['OI'] > 0:
+                    raw16 = infile.read(self.header['OI'] * 2)
+                if self.header['OL'] > 0:
+                    raw32 = infile.read(self.header['OL'] * 4)
 
-        else:
-            bytecode = numpy.int32
-            self.bpp = len(numpy.array(0, bytecode).tostring())
-            ReadBytes = self.dim1 * self.dim2 * self.bpp
-            block = numpy.fromstring(infile.read(ReadBytes), bytecode)
+                # endianess is handled at the decompression level
+                raw_data = decTY1(raw8, raw16, raw32)
+                bytecode = raw_data.dtype
+            elif self.header['Compression'] == 'TY5':
+                logger.warning("# Compressed with the TY5 compression")
+                bytecode = numpy.int8
+                self.bpp = 1
+                raw8 = infile.read(self.dim1 * self.dim2)
+                raw_data = numpy.fromstring(raw8, bytecode)
 
-        logger.debug('OVER_SHORT2: %s', block.dtype)
-        logger.debug("%s" % (block < 0).sum())
-        infile.close()
+                if self.header['OI'] > 0:
+                    self.raw16 = infile.read(self.header['OI'] * 2)
+                else:
+                    self.raw16 = ""
+                if self.header['OL'] > 0:
+                    self.raw32 = infile.read(self.header['OL'] * 4)
+                else:
+                    self.raw32 = ""
+                self.rest = infile.read()
+                self.blob = raw8 + self.raw16 + self.raw32 + self.rest
+                raw_data = self.dec_TY5(raw8 + self.raw16 + self.raw32)
+            else:
+                bytecode = numpy.int32
+                self.bpp = len(numpy.array(0, bytecode).tostring())
+                nbytes = self.dim1 * self.dim2 * self.bpp
+                raw_data = numpy.fromstring(infile.read(nbytes), bytecode)
+                # Always assume littel-endian on the disk
+                if not numpy.little_endian:
+                    raw_data.byteswap(True)
+#         infile.close()
+
+        logger.debug('OVER_SHORT2: %s', raw_data.dtype)
+        logger.debug("%s" % (raw_data < 0).sum())
         logger.debug("BYTECODE: %s", bytecode)
-        self.data = block.reshape((self.dim2, self.dim1))
+        self.data = raw_data.reshape((self.dim2, self.dim1))
         self.bytecode = self.data.dtype.type
         self.pilimage = None
         return self
@@ -241,8 +288,7 @@ class OXDimage(fabioimage):
         """
         linesep = "\r\n"
         for key in DEFAULT_HEADERS:
-            if key not in self.header_keys:
-                self.header_keys.append(key)
+            if key not in self.header:
                 self.header[key] = DEFAULT_HEADERS[key]
 
         if "NX" not in self.header.keys() or "NY" not in self.header.keys():
@@ -264,7 +310,7 @@ class OXDimage(fabioimage):
 
             ascii_headers.append("TIME=%s" % time.ctime())
 
-        header = (linesep.join(ascii_headers)).ljust(256)
+        header = (linesep.join(ascii_headers)).ljust(256).encode("ASCII")
 
 
         NG = Section(self.header['General Section size in Byte'], self.header)
@@ -382,11 +428,13 @@ class OXDimage(fabioimage):
 
     def write(self, fname):
         """Write Oxford diffraction images: this is still beta
-        @param fname: output filename 
+        Only TY1 compressed images is currently possible
+        @param fname: output filename
         """
         datablock8, datablock16, datablock32 = compTY1(self.data)
         self.header["OI"] = len(datablock16) / 2
         self.header["OL"] = len(datablock32) / 4
+        self.header["Compression"] = "TY1"
         with self._open(fname, mode="wb") as outfile:
             outfile.write(self._writeheader())
             outfile.write(datablock8)
@@ -404,6 +452,58 @@ class OXDimage(fabioimage):
         else:
             return data.astype(int)
 
+    def dec_TY5(self, stream):
+        """
+        Attempt to decode TY5 compression scheme
+
+        @param stream: input stream
+        @return: 1D array with data
+        """
+        array_size = self.dim1 * self.dim2
+        stream_size = len(stream)
+        data = numpy.zeros(array_size)
+        raw = numpy.fromstring(stream, dtype="uint8")
+        pos_inp = pos_out = current = ex1 = ex2 = 0
+
+        while pos_inp < stream_size and pos_out < array_size:
+            if pos_out % self.dim2 == 0:
+                last = 0
+            else:
+                last = current
+            value = raw[pos_inp]
+            if value < 254:
+                # this is the normal case
+#               # 1 bytes encode one pixel
+                current = last + value - 127
+                pos_inp += 1
+            elif value == 254:
+                ex1 += 1
+                # this is the special case 1:
+                # if the marker 254 is found the next 2 bytes encode one pixel
+                value = raw[pos_inp + 1:pos_inp + 3].view("int16")
+                if not numpy.little_endian:
+                    value = value.byteswap(True)
+                current = last + value[0]
+                pos_inp += 3
+
+            elif value == 255:
+                # this is the special case 2:
+                # if the marker 255 is found the next 4 bytes encode one pixel
+                ex2 += 1
+                print('special case 32 bits.')
+                value = raw[pos_inp + 1:pos_inp + 5].view("int32")
+                if not numpy.little_endian:
+                    value = value.byteswap(True)
+                current = last + value[0]
+                pos_inp += 5
+            data[pos_out] = current
+            pos_out += 1
+
+        print("Exception encountered: 2:%s, 4:%s" % (ex1, ex2))
+        return data
+
+OXDimage = OxdImage
+
 class Section(object):
     """
     Small helper class for writing binary headers
@@ -415,10 +515,10 @@ class Section(object):
         """
         self.size = size
         self.header = dictHeader
-        self.lstChr = ["\x00"] * size
+        self.lstChr = bytearray(size)
         self._dictSize = {}
     def __repr__(self):
-        return "".join(self.lstChr)
+        return bytes(self.lstChr)
 
     def getSize(self, dtype):
         if not dtype in self._dictSize:
@@ -439,6 +539,8 @@ class Section(object):
             value = default
         if value is None:
             value = "\x00" * self.getSize(dtype)
-        else:
+        elif numpy.little_endian:
             value = numpy.array(value).astype(dtype).tostring()
+        else:
+            value = numpy.array(value).astype(dtype).byteswap().tostring()
         self.lstChr[offset:offset + self.getSize(dtype)] = value
