@@ -1,7 +1,30 @@
-#!/usr/bin/env python
-# coding: utf8
+# coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 """
 Authors: Gael Goret, Jerome Kieffer, ESRF, France
+
 Emails: gael.goret@esrf.fr, jerome.kieffer@esrf.fr
         Brian Richard Pauw <brian@stack.nl>
 
@@ -18,25 +41,25 @@ __authors__ = ["Gaël Goret", "Jérôme Kieffer", "Brian Pauw"]
 __contact__ = "gael.goret@esrf.fr"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__version__ = "17 Oct 2012"
+__version__ = "17/10/2012"
 
-from .fabioimage import fabioimage
+from .fabioimage import FabioImage
 import numpy, logging
 logger = logging.getLogger("binaryimage")
 
-class binaryimage(fabioimage):
+class BinaryImage(FabioImage):
     """
     This simple library has been made for manipulating exotic/unknown files format.
 
-    Binary files images are simple none-compressed 2D images only defined by their :
-    data-type, dimensions, byte order and offset
+    Binary files images are simple none-compressed 2D images only defined by their:
+        data-type, dimensions, byte order and offset
 
     if offset is set to a negative value, the image is read using the last data but n
     data in the file, skipping any header.
     """
 
     def __init__(self, *args, **kwargs):
-        fabioimage.__init__(self, *args, **kwargs)
+        FabioImage.__init__(self, *args, **kwargs)
 
     @staticmethod
     def swap_needed(endian):
@@ -74,7 +97,7 @@ class binaryimage(fabioimage):
             f.seek(offset)
         else:
             try:
-                f.seek(-size + offset + 1, 2) #seek from EOF backwards
+                f.seek(-size + offset + 1, 2)  # seek from EOF backwards
             except IOError:
                 logging.warn('expected datablock too large, please check bytecode settings: {}'.format(bytecode))
             except:
@@ -101,4 +124,4 @@ class binaryimage(fabioimage):
         with open(fname, mode="wb") as outfile:
             outfile.write(self.data.tostring())
 
-
+binaryimage = BinaryImage
