@@ -1,5 +1,27 @@
-#!/usr/bin/env python
-# coding: utf8
+# coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 """
 Authors: Jérôme Kieffer, ESRF
          email:jerome.kieffer@esrf.fr
@@ -7,22 +29,39 @@ Authors: Jérôme Kieffer, ESRF
 FabIO library containing compression and decompression algorithm for various
 """
 # get ready for python3
-from __future__ import with_statement, print_function
-
+from __future__ import absolute_import, print_function, with_statement, division
 __author__ = "Jérôme Kieffer"
 __contact__ = "jerome.kieffer@esrf.eu"
 __license__ = "GPLv3+"
+__date__ = "30/10/2015"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 
 
-import logging, struct, hashlib, base64, StringIO, sys
-if sys.version_info >= (3,):
-    str = bytes
+import sys
+import base64
+import hashlib
+import struct
+import logging
 logger = logging.getLogger("compression")
+
+try:
+    import six
+    if tuple(int(i) for i in six.__version__.split(".")[:2]) < (1, 8):
+        raise ImportError("Six version is too old")
+except ImportError:
+    from .third_party import six
+
+
+if six.PY2:
+    bytes = str
+
 import numpy
 
 try:
-    import gzip
+    if sys.version_info < (2, 7):
+        from .third_party import gzip
+    else:
+        import gzip
 except ImportError:
     logger.error("Unable to import gzip module: disabling gzip compression")
     gzip = None
@@ -39,6 +78,7 @@ except ImportError:
     logger.error("Unable to import zlib module: disabling zlib compression")
     zlib = None
 
+
 def md5sum(blob):
     """
     returns the md5sum of an object...
@@ -65,22 +105,22 @@ def decGzip(stream):
 
     if gzip is None:
         raise ImportError("gzip module is not available")
-    fileobj = StringIO.StringIO(stream)
+    fileobj = six.BytesIO(stream)
     try:
         rawData = gzip.GzipFile(fileobj=fileobj).read()
     except IOError:
         logger.warning("Encounter the python-gzip bug with trailing garbage, trying subprocess gzip")
         try:
-            #This is as an ugly hack against a bug in Python gzip
+            # This is as an ugly hack against a bug in Python gzip
             import subprocess
             sub = subprocess.Popen(["gzip", "-d", "-f"], stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
             rawData, err = sub.communicate(input=stream)
             logger.debug("Gzip subprocess ended with %s err= %s; I got %s bytes back" % (sub.wait(), err, len(rawData)))
-        except Exception, error: #IGNORE:W0703
+        except Exception as error:  # IGNORE:W0703
             logger.warning("Unable to use the subprocess gzip (%s). Is gzip available? " % error)
             for i in range(1, 513):
                 try:
-                    fileobj = StringIO.StringIO(stream[:-i])
+                    fileobj = six.BytesIO(stream[:-i])
                     rawData = gzip.GzipFile(fileobj=fileobj).read()
                 except IOError:
                     logger.debug("trying with %s bytes less, doesn't work" % i)
@@ -113,112 +153,9 @@ def decZlib(stream):
     return zlib.decompress(stream)
 
 
-def decByteOffet_python(stream, size):
-    """
-    Analyze a stream of char with any length of exception (2,4, or 8 bytes integers)
-
-    @param stream: string representing the compressed data
-    @param size: the size of the output array (of longInts)
-    @return: 1D-ndarray
-
-    """
-    logger.debug("CBF decompression using Python with Cython loops")
-    dataOut = numpy.zeros((size), dtype=numpy.int64)
-    i = 0
-    j = 0
-    last = 0
-    current = 0
-    while ((i < len(stream)) and (j < size)):
-        if (stream[i] == '\x80'):
-            if (stream[i + 1:i + 3] == "\x00\x80"):
-                if (stream[i + 3:i + 7] == "\x00\x00\x00\x80"):
-                    current = struct.unpack("<q", stream[i + 7:i + 15])[0]
-                    i += 15
-                else:
-                    current = struct.unpack("<i", stream[i + 3:i + 7])[0]
-                    i += 7
-            else:
-                current = struct.unpack("<h", stream[i + 1:i + 3])[0]
-                i += 3
-        else:
-            current = struct.unpack("<b", stream[i])[0]
-            i += 1
-        last += current
-        dataOut[j] = last
-        j += 1
-    return dataOut
-
-def decByteOffet_weave(stream, size):
-    """
-    Analyze a stream of char with any length of exception (2,4, or 8 bytes integers)
-
-    @param stream: string representing the compressed data
-    @param size: the size of the output array (of longInts)
-    @return: 1D-ndarray
-
-    """
-    logger.debug("CBF decompression using Weave")
-    try:
-        from scipy import weave
-        from scipy.weave import converters
-    except ImportError:
-        logger.warning("scipy.weave is not available, falling back on slow Numpy implementations")
-        return decByteOffet_numpy(stream, size)
-    dataIn = numpy.fromstring(stream, dtype="uint8")
-    n = dataIn.size
-    dataOut = numpy.zeros(size, dtype="int64")
-    codeC = """
-unsigned char key = 0x80;
-long   j = 0;
-long   last=0;
-long   current=0;
-for (int i=0; i< n; i++){
-   if (j>=size){
-   //printf("i= %i<%i, j=%i < size= %i %i\\n",i,n,j,size,dataIn(i));
-       break;
-  }
-  if (dataIn(i) == key){
-      if ( (dataIn(i+1)==0) and (dataIn(i+2)==key) ){
-          if ( (dataIn(i+3)==0) and (dataIn(i+4)==0) and (dataIn(i+5)==0) and (dataIn(i+6)==key) )  {
-               // 64 bits mode
-               char tmp = dataIn(i+14) ;
-               current = (long(tmp)<<56) | (long(dataIn(i+13))<<48) | (long(dataIn(i+12))<<40) | (long(dataIn(i+11))<<32) | (long(dataIn(i+10))<<24) | (long(dataIn(i+9))<<16) | (long(dataIn(i+8))<<8) | (long(dataIn(i+7)));
-//                printf("64 bit int at pos %i, %i, value=%ld \\n",i,j,current);
-               i+=14;
-           }else{
-               // 32 bits mode
-               char tmp = dataIn(i+6) ;
-               current = (long(tmp)<<24) | (long(dataIn(i+5))<<16) | (long(dataIn(i+4))<<8) | (long(dataIn(i+3)));
-//                printf("32 bit int at pos %i, %i, value=%ld was %i %i %i %i %i %i %i\\n",i,j,current,dataIn(i),dataIn(i+1),dataIn(i+2),dataIn(i+3),dataIn(i+4),dataIn(i+5),dataIn(i+6));
-//                printf("%ld %ld %ld %ld\\n",(long(tmp)<<24) , (long(dataIn(i+5))<<16) , (long(dataIn(i+4))<<8) ,long(dataIn(i+3)));
-               i+=6;
-           }
-      }else{
-           // 16 bit mode
-           char tmp = dataIn(i+2);
-           current = (long(tmp)<<8) | (long (dataIn(i+1)));
-//           printf("16 bit int at pos %i, %i, value=%ld was %i %i %i\\n",i,j,current,dataIn(i),dataIn(i+1),dataIn(i+2));
-           i+=2;
-      }
-  }else{
-      // 8 bit mode
-      char tmp = dataIn(i) ;
-      current= long(tmp) ;
-  }
-  last+=current;
-  dataOut(j)=last;
-  j++ ;
-}
-return_val=0;
-     """
-    rc = weave.inline(codeC, ["dataIn", "dataOut", "n", "size" ], verbose=2, type_converters=converters.blitz)
-    if rc != 0:
-        logger.warning("weave binary module return error code %s" % rc)
-    return dataOut
 
 
-
-def decByteOffet_numpy(stream, size=None):
+def decByteOffset_numpy(stream, size=None):
     """
     Analyze a stream of char with any length of exception:
                 2, 4, or 8 bytes integers
@@ -230,9 +167,9 @@ def decByteOffet_numpy(stream, size=None):
     """
     logger.debug("CBF decompression using Numpy")
     listnpa = []
-    key16 = "\x80"
-    key32 = "\x00\x80"
-    key64 = "\x00\x00\x00\x80"
+    key16 = b"\x80"
+    key32 = b"\x00\x80"
+    key64 = b"\x00\x00\x00\x80"
     shift = 1
     while True:
         idx = stream.find(key16)
@@ -243,23 +180,28 @@ def decByteOffet_numpy(stream, size=None):
 
         if stream[idx + 1:idx + 3] == key32:
             if stream[idx + 3:idx + 7] == key64:
-#                        long int 64 bits
-                listnpa.append(numpy.fromstring(stream[idx + 7:idx + 15],
-                                             dtype="int64"))
+                # 64 bits int
+                res = numpy.fromstring(stream[idx + 7:idx + 15], dtype="int64")
+                listnpa.append(res)
                 shift = 15
-            else: #32 bit int
-                listnpa.append(numpy.fromstring(stream[idx + 3:idx + 7],
-                                             dtype="int32"))
+            else:
+                # 32 bits int
+                res = numpy.fromstring(stream[idx + 3:idx + 7], dtype="int32")
+                listnpa.append(res)
                 shift = 7
         else:  # int16
-            listnpa.append(numpy.fromstring(stream[idx + 1:idx + 3],
-                                         dtype="int16"))
+            res = numpy.fromstring(stream[idx + 1:idx + 3], dtype="int16")
+            listnpa.append(res)
             shift = 3
         stream = stream[idx + shift:]
+    if not numpy.little_endian:
+        for res in listnpa:
+            if res.dtype != numpy.int8:
+                res.byteswap(True)
     return  (numpy.hstack(listnpa)).astype("int64").cumsum()
 
 
-def decByteOffet_cython(stream, size=None):
+def decByteOffset_cython(stream, size=None):
     """
     Analyze a stream of char with any length of exception:
                 2, 4, or 8 bytes integers
@@ -271,14 +213,17 @@ def decByteOffet_cython(stream, size=None):
     """
     logger.debug("CBF decompression using cython")
     try:
-        from fabio.byte_offset import analyseCython
-    except ImportError, error:
+        from .byte_offset import analyseCython
+    except ImportError as error:
         logger.error("Failed to import byte_offset cython module, falling back on numpy method")
-        return decByteOffet_numpy(stream, size)
+        return decByteOffset_numpy(stream, size)
     else:
         return analyseCython(stream, size)
 
-def compByteOffet_numpy(data):
+decByteOffset = decByteOffset_cython
+
+
+def compByteOffset_numpy(data):
     """
     Compress a dataset into a string using the byte_offet algorithm
 
@@ -299,25 +244,25 @@ def compByteOffet_numpy(data):
     else:
         byteswap = True
     start = 0
-    binary_blob = ""
+    binary_blob = b""
     for stop in exceptions:
         if stop - start > 0:
             binary_blob += delta[start:stop].astype("int8").tostring()
         exc = delta[stop]
-        if (exc > 2147483647) or (exc < -2147483647): #2**31-1
-            binary_blob += "\x80\x00\x80\x00\x00\x00\x80"
+        if (exc > 2147483647) or (exc < -2147483647):  # 2**31-1
+            binary_blob += b"\x80\x00\x80\x00\x00\x00\x80"
             if byteswap:
                 binary_blob += delta[stop:stop + 1].byteswap().tostring()
             else:
                 binary_blob += delta[stop:stop + 1].tostring()
-        elif (exc > 32767) or (exc < -32767): #2**15-1
-            binary_blob += "\x80\x00\x80"
+        elif (exc > 32767) or (exc < -32767):  # 2**15-1
+            binary_blob += b"\x80\x00\x80"
             if byteswap:
                 binary_blob += delta[stop:stop + 1].astype("int32").byteswap().tostring()
             else:
                 binary_blob += delta[stop:stop + 1].astype("int32").tostring()
-        else: #>127
-            binary_blob += "\x80"
+        else:  # >127
+            binary_blob += b"\x80"
             if byteswap:
                 binary_blob += delta[stop:stop + 1].astype("int16").byteswap().tostring()
             else:
@@ -327,24 +272,32 @@ def compByteOffet_numpy(data):
         binary_blob += delta[start:].astype("int8").tostring()
     return binary_blob
 
+compByteOffset = compByteOffset_numpy
+
 
 def decTY1(raw_8, raw_16=None, raw_32=None):
     """
     Modified byte offset decompressor used in Oxford Diffraction images
-    
+
+    Note: Always expect little endian data on the disk
+
     @param raw_8:  strings containing raw data with integer 8 bits
     @param raw_16: strings containing raw data with integer 16 bits
     @param raw_32: strings containing raw data with integer 32 bits
     @return: numpy.ndarray
-    
+
     """
     data = numpy.fromstring(raw_8, dtype="uint8").astype(int)
     data -= 127
     if raw_32 is not None:
-        int32 = numpy.fromstring(raw_32, dtype="int32").astype(int)
+        int32 = numpy.fromstring(raw_32, dtype="int32")
+        if not numpy.little_endian:
+            int32.byteswap(True)
         exception32 = numpy.nonzero(data == 128)
     if raw_16 is not None:
-        int16 = numpy.fromstring(raw_16, dtype="int16").astype(int)
+        int16 = numpy.fromstring(raw_16, dtype="int16")
+        if not numpy.little_endian:
+            int16.byteswap(True)
         exception16 = numpy.nonzero(data == 127)
         data[exception16] = int16
     if raw_32:
@@ -370,43 +323,51 @@ def compTY1(data):
     @return: 3-tuple of strings: raw_8,raw_16,raw_32 containing raw data with integer of the given size
 
     """
-    fdata = data.flatten()
+    fdata = data.ravel()
     diff = numpy.zeros_like(fdata)
     diff[0] = fdata[0]
     diff[1:] = fdata[1:] - fdata[:-1]
     adiff = abs(diff)
-    exception32 = (adiff > 32767)#2**15-1
-    exception16 = (adiff >= 127) - exception32 #2**7-1)
+    exception32 = (adiff > (1 << 15) - 1)
+    exception16 = (adiff >= (1 << 7) - 1) ^ exception32
     we16 = numpy.where(exception16)
     we32 = numpy.where(exception32)
-    raw_16 = diff[we16].astype("int16").tostring()
-    raw_32 = diff[we32].astype("int32").tostring()
+    data_16 = diff[we16].astype(numpy.int16)
+    data_32 = diff[we32].astype(numpy.int32)
+    if not numpy.little_endian:
+        data_16.byteswap(True)
+        data_32.byteswap(True)
     diff[we16] = 127
     diff[we32] = 128
     diff += 127
-    raw_8 = diff.astype("uint8").tostring()
-    return  raw_8, raw_16, raw_32
+    data_8 = diff.astype(numpy.uint8)
+    return  data_8.tostring(), data_16.tostring(), data_32.tostring()
 
-def decPCK(stream, dim1=None, dim2=None, overflowPix=None, version=None):
+
+def decPCK(stream, dim1=None, dim2=None, overflowPix=None, version=None, normal_start=None, swap_needed=None):
     """
     Modified CCP4  pck decompressor used in MAR345 images
 
-    @param stream: string or file
-    @return: numpy.ndarray (square array)
+    @param raw: input string (bytes in python3)
+    @param dim1,dim2: optional parameters size
+    @param overflowPix: optional parameters: number of overflowed pixels
+    @param version: PCK version 1 or 2
+    @param normal_start: position of the normal value section (can be auto-guessed)
+    @param swap_needed: set to True when reading data from a foreign endianness (little on big or big on little)
+    @return : ndarray of 2D with the right size
 
     """
-
     try:
-        from mar345_IO import uncompress_pck
-    except ImportError, error:
+        from .mar345_IO import uncompress_pck
+    except ImportError as  error:
         raise RuntimeError("Unable to import mar345_IO to read compressed dataset")
     if "seek" in dir(stream):
         stream.seek(0)
         raw = stream.read()
     else:
-        raw = str(stream)
+        raw = bytes(stream)
 
-    return uncompress_pck(raw, dim1, dim2, overflowPix, version)
+    return uncompress_pck(raw, dim1, dim2, overflowPix, version, normal_start, swap_needed)
 
 
 def compPCK(data):
@@ -418,8 +379,8 @@ def compPCK(data):
 
     """
     try:
-        from mar345_IO import compress_pck
-    except ImportError, error:
+        from .mar345_IO import compress_pck
+    except ImportError as error:
         raise RuntimeError("Unable to import mar345_IO to write compressed dataset")
     return compress_pck(data)
 
