@@ -1,4 +1,26 @@
-#!/usr/bin/env python
+# coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 
 """
 
@@ -14,7 +36,11 @@ Authors: Henning O. Sorensen & Erik Knudsen
 """
 # get ready for python3
 from __future__ import with_statement, print_function, absolute_import, division
-import os, logging, sys, tempfile, threading
+import os
+import logging
+import sys
+import tempfile
+import threading
 logger = logging.getLogger("fabioimage")
 import numpy
 try:
@@ -24,8 +50,36 @@ except ImportError:
     Image = None
 from . import fabioutils, converters
 
+try:
+    import six
+    six_version = tuple(int(i) for i in six.__version__.split() if i.isdigit())
+    if six_version < (1, 8):
+        for i in ("six", "six.moves"):
+            sys.modules.pop(i, None)
+        raise ImportError("Old version")
+    from six import with_metaclass
+except ImportError:
+    from .third_party.six import with_metaclass
 
-class fabioimage(object):
+try:
+    from collections import OrderedDict
+except ImportError:
+    from .third_party.ordereddict import OrderedDict
+
+
+class FabioMeta(type):
+    """
+    Metaclass used to register all image classes inheriting from fabioImage
+    """
+    # we use __init__ rather than __new__ here because we want
+    # to modify attributes of the class *after* they have been
+    # created
+    def __init__(cls, name, bases, dct):
+        cls.registry[name.lower()] = cls
+        super(FabioMeta, cls).__init__(name, bases, dct)
+
+
+class FabioImage(with_metaclass(FabioMeta, object)):
     """
     A common object for images in fable
     Contains a numpy array (.data) and dict of meta data (.header)
@@ -33,28 +87,42 @@ class fabioimage(object):
 
     _need_a_seek_to_read = False
     _need_a_real_file = False
+    registry = OrderedDict()  # list of child classes ...
+
+    @classmethod
+    def factory(cls, name):
+        """
+        A kind of factory...
+
+        @param name: name of the class to instanciate
+        @type name: str
+        @return: an instance of the class
+        @rtype: fabioimage
+        """
+        name = name.lower()
+        obj = None
+        if name in cls.registry:
+            obj = cls.registry[name]()
+        else:
+            msg = ("FileType %s is unknown !, "
+                   "please check if the filename exists or select one from %s" % (name, cls.registry.keys()))
+            logger.error(msg)
+            raise RuntimeError(msg)
+        return obj
 
     def __init__(self, data=None , header=None):
         """
         Set up initial values
         """
         self._classname = None
+        self._dim1 = self._dim2 = self._bpp = 0
+        self._bytecode = None
         if type(data) in fabioutils.StringTypes:
             raise Exception("fabioimage.__init__ bad argument - " + \
                             "data should be numpy array")
-        self.data = self.checkData(data)
+        self.data = self.check_data(data)
         self.pilimage = None
-        if header is None:
-            self.header = {}
-        else:
-            self.header = self.checkHeader(header)
-        self.header_keys = list(self.header.keys())  # holds key ordering
-        if self.data is not None:
-            self.dim2, self.dim1 = self.data.shape
-        else:
-            self.dim1 = self.dim2 = 0
-        self.bytecode = None  # numpy typecode
-        self.bpp = 2  # bytes per pixel
+        self.header = self.check_header(header)
         # cache for image statistics
         self.mean = self.maxval = self.stddev = self.minval = None
         # Cache roi
@@ -67,22 +135,89 @@ class fabioimage(object):
         self.filename = None
         self.filenumber = None
 
-    @staticmethod
-    def checkHeader(header=None):
-        """
-        Empty for fabioimage but may be populated by others classes
-        """
-        if header is None:
-            return {}
+    def get_header_keys(self):
+        return list(self.header.keys())
+    def set_header_keys(self, value):
+        pass
+    header_keys = property(get_header_keys, set_header_keys)
+
+    def get_dim1(self):
+        "Getter for dim1: data superseeds _dim1"
+        if self.data is not None:
+            try:
+                return self.data.shape[1]
+            except IndexError as err:
+                print(err)
+                print(self.data)
+                return self._dim1
         else:
-            return header
+            return self._dim1
+    def set_dim1(self, value):
+        "Setter for dim1"
+        self._dim1 = value
+    dim1 = property(get_dim1, set_dim1)
+
+    def get_dim2(self):
+        "Getter for dim2: data superseeds _dim2"
+        if self.data is not None:
+            return self.data.shape[0]
+        else:
+            return self._dim2
+    def set_dim2(self, value):
+        "Setter for dim2"
+        self._dim2 = value
+    dim2 = property(get_dim2, set_dim2)
+
+    def get_bpp(self):
+        "Getter for bpp: data superseeds _bpp"
+        if self.data is not None:
+            return numpy.dtype(self.data.dtype).itemsize
+        elif self._bytecode is not None:
+            return numpy.dtype(self._bytecode).itemsize
+        else:
+            return self._bpp
+    def set_bpp(self, value):
+        "Setter for bpp"
+        self._bpp = value
+    bpp = property(get_bpp, set_bpp)
+
+    def get_bytecode(self):
+        "Getter for bpp: data superseeds _bytecode"
+        if self.data is not None:
+            return self.data.dtype.type
+        else:
+            return self._bytecode
+    def set_bytecode(self, value):
+        "Setter for bpp"
+        self._bytecode = value
+    bytecode = property(get_bytecode, set_bytecode)
 
     @staticmethod
-    def checkData(data=None):
+    def check_header(header=None):
         """
-        Empty for fabioimage but may be populated by others classes, especially for format accepting only integers
+        Empty for fabioimage but may be populated by others classes
+
+        @param header: dict like object
+        @return: Ordered dict
         """
-        return data
+        if header is None:
+            return OrderedDict()
+        else:
+            return OrderedDict(header)
+
+    @staticmethod
+    def check_data(data=None):
+        """
+        Empty for fabioimage but may be populated by others classes,
+        especially for format accepting only integers
+
+        @param data: array like
+        @return: numpy array or None
+        """
+        if data is None:
+            return None
+        else:
+            return data
 
     def getclassname(self):
         """
@@ -151,13 +286,15 @@ class fabioimage(object):
     def getmax(self):
         """ Find max value in self.data, caching for the future """
         if self.maxval is None:
-            self.maxval = self.data.max()
+            if self.data is not None:
+                self.maxval = self.data.max()
         return self.maxval
 
     def getmin(self):
         """ Find min value in self.data, caching for the future """
         if self.minval is None:
-            self.minval = self.data.min()
+            if self.data is not None:
+                self.minval = self.data.min()
         return self.minval
 
     def make_slice(self, coords):
@@ -261,7 +398,7 @@ class fabioimage(object):
         else:
             dataIn = self.data.astype("float64")
             shapeIn = self.data.shape
-            shapeOut = (shapeIn[0] / y_rebin_fact, shapeIn[1] / x_rebin_fact)
+            shapeOut = (shapeIn[0] // y_rebin_fact, shapeIn[1] // x_rebin_fact)
             binsize = y_rebin_fact * x_rebin_fact
             if binsize < 50:  # method faster for small binning (4x4)
                 out = numpy.zeros(shapeOut, dtype="float64")
@@ -318,9 +455,6 @@ class fabioimage(object):
         by default pass in a dict of key, values.
         """
         self.header.update(kwds)
-        for key in kwds:
-            if key not in self.header_keys:
-                 self.header_keys.append(key)
 
     def read(self, filename, frame=None):
         """
@@ -360,14 +494,14 @@ class fabioimage(object):
         if hasattr(fname, "read") and hasattr(fname, "write"):
             # It is already something we can use
             if "name" in dir(fname):
-                self.header["filename"] =self.filename= fname.name
+                self.filename = fname.name
             else:
-                self.filename = self.header["filename"] = "stream"
+                self.filename = "stream"
                 try:
                     setattr(fname, "name", self.filename)
                 except AttributeError:
-                    #cStringIO
-                    logger.warning("Unable to set filename attribute to stream (cStringIO?) of type %s"%type(fname))
+                    # cStringIO
+                    logger.warning("Unable to set filename attribute to stream (cStringIO?) of type %s" % type(fname))
             return fname
 
         fileObject = None
@@ -375,7 +509,6 @@ class fabioimage(object):
         self.filenumber = fabioutils.extract_filenumber(fname)
 
         if isinstance(fname, fabioutils.StringTypes):
-            self.header["filename"] = fname
             if os.path.splitext(fname)[1] == ".gz":
                 fileObject = self._compressed_stream(fname,
                                        fabioutils.COMPRESSORS['.gz'],
@@ -412,11 +545,10 @@ class fabioimage(object):
         fobj = None
         if self._need_a_real_file and mode[0] == "r":
             fo = python_uncompress(fname, mode)
-#            fobj = os.tmpfile()
             # problem when not administrator under certain flavors of windows
             tmpfd, tmpfn = tempfile.mkstemp()
             os.close(tmpfd)
-            fobj = fabioutils.File(tmpfn, "w+b")
+            fobj = fabioutils.File(tmpfn, "w+b", temporary=True)
             fobj.write(fo.read())
             fo.close()
             fobj.seek(0)
@@ -431,123 +563,34 @@ class fabioimage(object):
         """
         Convert a fabioimage object into another fabioimage object (with possible conversions)
         @param dest: destination type "EDF", "edfimage" or the class itself
+        @return: instance of the new class
         """
+        other = None
         if type(dest) in fabioutils.StringTypes:
             dest = dest.lower()
-            modules = []
-            for val  in fabioutils.FILETYPES.values():
-                modules += [i + "image" for i in val if i not in modules]
-            klass = None
-            module = None
-            klass_name = None
-            for klass_name in modules:
-                if  klass_name.startswith(dest):
-                    try:
-                        module = sys.modules["fabio." + klass_name]
-                    except KeyError:
-                        try:
-                            module = __import__(klass_name)
-                        except:
-                            logger.error("Failed to import %s", klass_name)
-                        else:
-                            logger.debug("imported %simage", klass_name)
-                    if module is not None:
-                        break
-            if module is not None:
-                if hasattr(module, klass_name):
-                    klass = getattr(module, klass_name)
+            if dest.endswith("image"):
+                dest = dest[:-5]
+            if dest + "image" in self.registry:
+                other = self.factory(dest + "image")
+            # load modules which could be suitable:
+            for pref in fabioutils.FILETYPES.get(dest, []):
+                try:
+                    __import__(".%simage" % pref)
+                    other = self.factory(pref + "image")
+                except:
+                    pass
                 else:
-                    logger.error("Module %s has no image class" % module)
+                    continue
+
         elif isinstance(dest, self.__class__):
-            klass = dest.__class__
+            other = dest.__class__()
         elif ("__new__" in dir(dest)) and isinstance(dest(), fabioimage):
-            klass = dest
+            other = dest()
         else:
-            logger.warning("Unrecognized destination format: %s " % dest)
+            logger.error("Unrecognized destination format: %s " % dest)
             return self
-        if klass is None:
-            logger.warning("Unrecognized destination format: %s " % dest)
-            return self
-        other = klass()  # temporary instance (to be overwritten)
-        other = klass(data=converters.convert_data(self.classname, other.classname, self.data),
-                    header=converters.convert_header(self.classname, other.classname, self.header))
+        other.data = converters.convert_data(self.classname, other.classname, self.data)
+        other.header = converters.convert_header(self.classname, other.classname, self.header)
         return other
 
-def test():
-    """
-    check some basic fabioimage functionality
-    """
-    import time
-    start = time.time()
-
-    dat = numpy.ones((1024, 1024), numpy.uint16)
-    dat = (dat * 50000).astype(numpy.uint16)
-    assert dat.dtype.char == numpy.ones((1), numpy.uint16).dtype.char
-    hed = {"Title":"50000 everywhere"}
-    obj = fabioimage(dat, hed)
-
-    assert obj.getmax() == 50000
-    assert obj.getmin() == 50000
-    assert obj.getmean() == 50000 , obj.getmean()
-    assert obj.getstddev() == 0.
-
-    dat2 = numpy.zeros((1024, 1024), numpy.uint16, savespace=1)
-    cord = [ 256, 256, 790, 768 ]
-    slic = obj.make_slice(cord)
-    dat2[slic] = dat2[slic] + 100
-
-    obj = fabioimage(dat2, hed)
-
-    # New object, so...
-    assert obj.maxval is None
-    assert obj.minval is None
-
-    assert obj.getmax() == 100, obj.getmax()
-    assert obj.getmin() == 0 , obj.getmin()
-    npix = (slic[0].stop - slic[0].start) * (slic[1].stop - slic[1].start)
-    obj.resetvals()
-    area1 = obj.integrate_area(cord)
-    obj.resetvals()
-    area2 = obj.integrate_area(slic)
-    assert area1 == area2
-    assert obj.integrate_area(cord) == obj.integrate_area(slic)
-    assert obj.integrate_area(cord) == npix * 100, obj.integrate_area(cord)
-
-
-    def clean():
-        """ clean up the created testfiles"""
-        for name in ["testfile", "testfile.gz", "testfile.bz2"]:
-            try:
-                os.remove(name)
-            except:
-                continue
-
-
-    clean()
-    import gzip, bz2
-    gzip.open("testfile.gz", "wb").write("{ hello }")
-    fout = obj._open("testfile.gz")
-    readin = fout.read()
-    assert readin == "{ hello }", readin + " gzipped file"
-
-
-    bz2.BZ2File("testfilebz", "wb").write("{ hello }")
-    fout = obj._open("testfile.bz2")
-    readin = fout.read()
-    assert readin == "{ hello }", readin + " bzipped file"
-
-    ftest = open("testfile", "wb")
-    ftest.write("{ hello }")
-    assert ftest == obj._open(ftest)
-    ftest.close()
-    fout = obj._open("testfile")
-    readin = fout.read()
-    assert readin == "{ hello }", readin + "plain file"
-    fout.close()
-    ftest.close()
-    clean()
-
-    print ("Passed in %s s" % (time.time() - start))
-
-if __name__ == '__main__':
-    test()
+fabioimage = FabioImage
