@@ -1,15 +1,35 @@
-#!/usr/bin/env python
 # coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 from __future__ import absolute_import, print_function, with_statement, division
 
-__doc__ = """
-General purpose utilities functions for fabio
-"""
+__doc__ = """General purpose utilities functions for fabio"""
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
-__license__ = "GPL"
+__license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "08/01/2015"
+__date__ = "04/11/2015"
 __status__ = "stable"
 __docformat__ = 'restructuredtext'
 
@@ -27,7 +47,7 @@ if six.PY2:
     bytes = str
     FileIO = file
     StringTypes = (str, unicode)
-    to_str = lambda s: str(s)
+    to_str = str
 else:
     StringTypes = (str, bytes)
     unicode = str
@@ -38,7 +58,7 @@ from .compression import bz2, gzip
 import traceback
 from math import ceil
 
-if sys.version_info < (3, 4):
+if sys.version_info < (3, 3):
     from threading import _Semaphore as _Semaphore
 else:
     from threading import Semaphore as _Semaphore
@@ -370,7 +390,7 @@ def toAscii(name, excluded=None):
             if i in ascii:
                 ascii.remove(i)
             else:
-                logger.error("toAscii: % not in ascii table" % i)
+                logger.error("toAscii: %s not in ascii table" % i)
         dictAscii[excluded] = ascii
     else:
         ascii = dictAscii[excluded]
@@ -425,7 +445,7 @@ class File(FileIO):
     """
     wrapper for "file" with locking
     """
-    def __init__(self, name, mode="rb", buffering=0):
+    def __init__(self, name, mode="rb", buffering=0, temporary=False):
         """file(name[, mode[, buffering]]) -> file object
 
         Open a file.  The mode can be 'r', 'w' or 'a' for reading (default),
@@ -443,6 +463,8 @@ class File(FileIO):
         '\r', '\n', '\r\n' or a tuple containing all the newline types seen.
 
         'U' cannot be combined with 'w' or '+' mode.
+        
+        @param temporary: if True, destroy file at close.
         """
         if six.PY2:
             FileIO.__init__(self, name, mode, buffering)
@@ -450,6 +472,17 @@ class File(FileIO):
             FileIO.__init__(self, name, mode)
         self.lock = _Semaphore()
         self.__size = None
+        self.__temporary = temporary
+
+    def close(self):
+        name = self.name
+        FileIO.close(self)
+        if self.__temporary:
+            try:
+                os.unlink(name)
+            except Exception as err:
+                logger.error("Unable to remove %s: %s" % (name, err))
+                raise(err)
 
     def getSize(self):
         if self.__size is None:
@@ -529,10 +562,13 @@ else:
             if self.__size is None:
                 with self.lock:
                     if self.__size is None:
-                        pos = self.offset
+                        if "offset" in dir(self):
+                            pos = self.offset
+                        elif "tell" in dir(self):
+                            pos = self.tell()
                         end_pos = len(gzip.GzipFile.read(self)) + pos
                         self.seek(pos)
-                        logger.debug("Measuring size of %s: %s @ %s == %s" % (self.name, end_pos, pos, self.offset))
+                        logger.debug("Measuring size of %s: %s @ %s == %s" % (self.name, end_pos, pos, pos))
                         self.__size = end_pos
             return self.__size
 
