@@ -1,4 +1,26 @@
-#!/usr/bin/env python
+# coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 
 """
 
@@ -15,14 +37,14 @@ Authors:
 
 """
 # Get ready for python3:
-from __future__ import with_statement, print_function
+from __future__ import absolute_import, print_function, with_statement, division
 import logging, sys
 logger = logging.getLogger("fileseries")
 import traceback as pytraceback
 
-from fabioutils import FilenameObject, next_filename
+from .fabioutils import FilenameObject, next_filename
 
-from openimage import openimage
+from .openimage import openimage
 
 
 def new_file_series0(first_object, first=None, last=None, step=1):
@@ -46,14 +68,14 @@ def new_file_series0(first_object, first=None, last=None, step=1):
         try:
             newim = im.next()
             im = newim
-        except Exception, error:
+        except Exception as error:
             pytraceback.print_exc()
 
             # Skip bad images
             logger.warning("Got a problem here: %s", error)
             try:
                 im.filename = next_filename(im.filename)
-            except Exception, error:
+            except Exception as error:
                 # KE: This will not work and will throw an exception
                 # fabio.next_filename doesn't understand %nnnn on the end
                 logger.warning("Got another problem here: %s", error)
@@ -84,9 +106,9 @@ def new_file_series(first_object, nimages=0, step=1, traceback=False):
         from which all the exception information can be obtained.
 
     Suggested usage:
-    
+
     ::
-    
+
         for obj in new_file_series( ... ):
           if not isinstance(obj, fabio.fabioimage.fabioimage ):
             # deal with errors like missing images, non readable files, etc
@@ -105,7 +127,7 @@ def new_file_series(first_object, nimages=0, step=1, traceback=False):
             newim = im.next()
             im = newim
             retVal = im
-        except Exception, ex:
+        except Exception as ex:
             retVal = sys.exc_info()
             if(traceback):
                 pytraceback.print_exc()
@@ -114,7 +136,7 @@ def new_file_series(first_object, nimages=0, step=1, traceback=False):
             # Skip bad images
             try:
                 im.filename = next_filename(im.filename)
-            except Exception, ex:
+            except Exception as ex:
                 logger.warning("Got another problem here: next_filename(im.filename) %s", ex)
         if nprocessed % step == 0:
             yield retVal
