@@ -1,5 +1,27 @@
-#!/usr/bin/env python
 # coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 
 """
 HDF5 image for FabIO
@@ -23,7 +45,7 @@ __copyright__ = "Jérôme Kieffer"
 __version__ = "15/02/2015"
 
 import numpy, logging, os, posixpath, sys, copy
-from .fabioimage import fabioimage
+from .fabioimage import FabioImage
 logger = logging.getLogger("hdf5image")
 if sys.version_info[0] < 3:
     bytes = str
@@ -38,7 +60,7 @@ except ImportError:
 from .fabioutils import previous_filename, next_filename
 
 
-class hdf5image(fabioimage):
+class Hdf5Image(FabioImage):
     """
     FabIO image class for Images from an HDF file
     """
@@ -47,11 +69,11 @@ class hdf5image(fabioimage):
         Generic constructor
         """
         if not h5py:
-            raise RuntimeError("fabio.hdf5image cannot be used without h5py. Please install h5py and restart")
+            raise RuntimeError("fabio.Hdf5Image cannot be used without h5py. Please install h5py and restart")
 
-        fabioimage.__init__(self, *arg, **kwargs)
+        FabioImage.__init__(self, *arg, **kwargs)
         self.data = None
-        self.header = {}
+        self.header = self.check_header()
         self.dim1 = self.dim2 = 0
         self.m = self.maxval = self.stddev = self.minval = None
         self.header_keys = self.header.keys()
@@ -124,13 +146,13 @@ class hdf5image(fabioimage):
 
     def getframe(self, num):
         """
-        Returns a frame as a new fabioimage object
+        Returns a frame as a new FabioImage object
         @param num: frame number
         """
         if num < 0 or num > self.nframes:
             raise RuntimeError("Requested frame number is out of range")
         # Do a deep copy of the header to make a new one
-        frame = hdf5image(header=self.header.copy())
+        frame = Hdf5Image(header=self.header.copy())
         frame.header_keys = self.header_keys[:]
         for key in ("dim1", "dim2", "nframes", "bytecode", "hdf5", "ds"):
             frame.__setattr__(key, self.__getattribute__(key))
@@ -150,7 +172,7 @@ class hdf5image(fabioimage):
         if self.currentframe < (self.nframes - 1) and self.nframes > 1:
             return self.getframe(self.currentframe + 1)
         else:
-            newobj = hdf5image()
+            newobj = Hdf5Image()
             newobj.read(next_filename(self.filename))
             return newobj
 
@@ -161,6 +183,9 @@ class hdf5image(fabioimage):
         if self.currentframe > 0:
             return self.getframe(self.currentframe - 1)
         else:
-            newobj = hdf5image()
+            newobj = Hdf5Image()
             newobj.read(previous_filename(self.filename))
             return newobj
+
+
+hdf5image = Hdf5Image
