@@ -1,5 +1,27 @@
-#!/usr/bin/env python
-#coding: utf-8
+# coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 
 """
 Author: Jon Wright, ESRF.
@@ -11,20 +33,20 @@ __authors__ = ["Jon Wright", "Jérôme Kieffer"]
 __contact__ = "wright@esrf.fr"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__version__ = "29 Oct 2013"
+__date__ = "30/10/2015"
 
 import numpy
 import sys
 import os
-from .fabioimage import fabioimage
+from .fabioimage import FabioImage
 from .fabioutils import previous_filename, next_filename
 
-class pixiimage(fabioimage):
+class PixiImage(FabioImage):
     _need_a_seek_to_read = True
 
     def _readheader(self, infile):
         infile.seek(0)
-        self.header = {}
+        self.header = self.check_header()
         byt = infile.read(4)
         framesize = numpy.fromstring(byt, numpy.int32)
         if framesize == 243722:
@@ -43,7 +65,7 @@ class pixiimage(fabioimage):
     def read(self, fname, frame=None):
         if frame is None:
             frame = 0
-        self.header = {}
+        self.header = self.check_header()
         self.resetvals()
         infile = self._open(fname, "rb")
         self.sequencefilename = fname
@@ -74,7 +96,7 @@ class pixiimage(fabioimage):
 
     def getframe(self, num):
         """
-        Returns a frame as a new fabioimage object
+        Returns a frame as a new FabioImage object
         """
         if num < 0 or num > self.nframes:
             raise Exception("Requested frame number is out of range")
@@ -116,9 +138,11 @@ class pixiimage(fabioimage):
             return newobj
 
 
+pixiimage = PixiImage
+
 
 def demo(fname):
-    i = pixiimage()
+    i = PixiImage()
     i.read(fname)
     import pylab
     pylab.imshow(numpy.log(i.data))
