@@ -1,40 +1,47 @@
 #!/usr/bin/env python
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
+#
+#    Project: Fable Input Output
+#             https://github.com/kif/fabio
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 """
 # Unit tests
 
 # builds on stuff from ImageD11.test.testpeaksearch
 """
 
+from __future__ import print_function, with_statement, division, absolute_import
 import unittest
-import os
-import logging
 import sys
-logger = logging.getLogger("testXSDimage")
-force_build = False
+import os
+if __name__ == '__main__':
+    import pkgutil
+    __path__ = pkgutil.extend_path([os.path.dirname(__file__)], "fabio.test")
+from .utilstest import UtilsTest
 
-for opts in sys.argv[:]:
-    if opts in ["-d", "--debug"]:
-        logging.basicConfig(level=logging.DEBUG)
-        sys.argv.pop(sys.argv.index(opts))
-    elif opts in ["-i", "--info"]:
-        logging.basicConfig(level=logging.INFO)
-        sys.argv.pop(sys.argv.index(opts))
-    elif opts in ["-f", "--force"]:
-        force_build = True
-        sys.argv.pop(sys.argv.index(opts))
-try:
-    logger.debug("Tests loaded from file: %s" % __file__)
-except:
-    __file__ = os.getcwd()
+logger = UtilsTest.get_logger(__file__)
+fabio = sys.modules["fabio"]
 
-from utilstest import UtilsTest
-if force_build:
-    UtilsTest.forceBuild()
-import fabio
 from fabio.xsdimage import xsdimage
 import numpy
-# filename dim1 dim2 min max mean stddev values are from OD Sapphire 3.0 
+# filename dim1 dim2 min max mean stddev values are from OD Sapphire 3.0
 TESTIMAGES = """XSDataImage.xml     512 512        86 61204     511.63    667.15
                 XSDataImageInv.xml  512 512  -0.2814 0.22705039 2.81e-08  0.010"""
 
@@ -78,19 +85,18 @@ class testXSD(unittest.TestCase):
         0, 3, "matrices are invert of each other")
 
 
-def test_suite_all_XSD():
-    testSuite = unittest.TestSuite()
+def suite():
+    testsuite = unittest.TestSuite()
     if xsdimage is None:
-        logging.warning("xsdimage is None ... probably an import error related to lxml. Skipping test")
+        logger.warning("xsdimage is None ... probably an import error related to lxml. Skipping test")
     else:
-        testSuite.addTest(testXSD("test_read"))
-        testSuite.addTest(testXSD("test_same"))
-        testSuite.addTest(testXSD("test_invert"))
-    return testSuite
+        testsuite.addTest(testXSD("test_read"))
+        testsuite.addTest(testXSD("test_same"))
+        testsuite.addTest(testXSD("test_invert"))
+    return testsuite
 
 
 if __name__ == '__main__':
-    mysuite = test_suite_all_XSD()
     runner = unittest.TextTestRunner()
-    runner.run(mysuite)
+    runner.run(suite())
 
