@@ -1,5 +1,26 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+#
+#    Project: Fable Input Output
+#             https://github.com/kif/fabio
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 """
 Test cases for the flat binary images
 
@@ -10,14 +31,12 @@ from __future__ import print_function, with_statement, division, absolute_import
 import unittest
 import sys
 import os
-import numpy
-import gzip
-import bz2
 
-try:
-    from .utilstest import UtilsTest
-except (ValueError, SystemError):
-    from utilstest import UtilsTest
+
+if __name__ == '__main__':
+    import pkgutil
+    __path__ = pkgutil.extend_path([os.path.dirname(__file__)], "fabio.test")
+from .utilstest import UtilsTest
 
 logger = UtilsTest.get_logger(__file__)
 fabio = sys.modules["fabio"]
@@ -61,13 +80,13 @@ class TestFlatBinary(unittest.TestCase):
             os.remove(filename)
 
 
-def test_suite_all_flat():
-    testSuite = unittest.TestSuite()
+def suite():
+    testsuite = unittest.TestSuite()
 
-#     testSuite.addTest(TestFlatBinary("test_openimage"))
-    return testSuite
+#     testsuite.addTest(TestFlatBinary("test_openimage"))
+    return testsuite
 
 if __name__ == '__main__':
-    mysuite = test_suite_all_flat()
     runner = unittest.TextTestRunner()
-    runner.run(mysuite)
+    runner.run(suite())
+
