@@ -1,5 +1,26 @@
 #!/usr/bin/env python
-# coding: utf8
+# -*- coding: utf-8 -*-
+#
+#    Project: Fable Input Output
+#             https://github.com/kif/fabio
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 
 """
 2011: Jerome Kieffer for ESRF.
@@ -7,54 +28,45 @@
 Unit tests for CBF images based on references images taken from:
 http://pilatus.web.psi.ch/DATA/DATASETS/insulin_0.2/
 
+19/01/2015
 """
-import unittest, sys, os, logging, tempfile
-logger = logging.getLogger("testcbfimage")
-force_build = False
-
-for opts in sys.argv[:]:
-    if opts in ["-d", "--debug"]:
-        logging.basicConfig(level=logging.DEBUG)
-        sys.argv.pop(sys.argv.index(opts))
-    elif opts in ["-i", "--info"]:
-        logging.basicConfig(level=logging.INFO)
-        sys.argv.pop(sys.argv.index(opts))
-    elif opts in ["-f", "--force"]:
-        force_build = True
-        sys.argv.pop(sys.argv.index(opts))
-try:
-    logger.debug("Tests loaded from file: %s" % __file__)
-except:
-    __file__ = os.getcwd()
-
-from utilstest import UtilsTest
-if force_build:
-    UtilsTest.forceBuild()
-import fabio
-from fabio.cbfimage import cbfimage
-from fabio.compression import decByteOffet_numpy, decByteOffet_cython
+from __future__ import print_function, with_statement, division, absolute_import
+import unittest
+import sys
+import os
+import numpy
+import gzip
+import bz2
 import time
 
-class test_cbfimage_reader(unittest.TestCase):
+if __name__ == '__main__':
+    import pkgutil
+    __path__ = pkgutil.extend_path([os.path.dirname(__file__)], "fabio.test")
+from .utilstest import UtilsTest
+
+logger = UtilsTest.get_logger(__file__)
+fabio = sys.modules["fabio"]
+from fabio.cbfimage import cbfimage
+from fabio.compression import decByteOffset_numpy, decByteOffset_cython
+from fabio.third_party.six import PY3
+if PY3:
+    from fabio.fabioutils import unicode
+
+
+class TestCbfReader(unittest.TestCase):
     """ test cbf image reader """
 
     def __init__(self, methodName):
         "Constructor of the class"
         unittest.TestCase.__init__(self, methodName)
-        testimgdir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "testimages")
-        self.edf_filename = os.path.join(testimgdir, "run2_1_00148.edf")
-        self.cbf_filename = os.path.join(testimgdir, "run2_1_00148.cbf")
-
+        self.edf_filename = os.path.join(UtilsTest.image_home, "run2_1_00148.edf")
+        self.cbf_filename = os.path.join(UtilsTest.image_home, "run2_1_00148.cbf")
 
     def setUp(self):
         """Download images"""
 
         UtilsTest.getimage(os.path.basename(self.edf_filename + ".bz2"))
         UtilsTest.getimage(os.path.basename(self.cbf_filename + ".bz2"))
-        self.tempdir = tempfile.mkdtemp()
-    def tearDown(self):
-        UtilsTest.recursive_delete(self.tempdir)
-
 
     def test_read(self):
         """ check whole reader"""
@@ -73,43 +85,31 @@ class test_cbfimage_reader(unittest.TestCase):
         name = os.path.basename(self.cbf_filename)
         obj = cbfimage()
         obj.read(self.cbf_filename)
-        obj.write(os.path.join(self.tempdir, name))
+        obj.write(os.path.join(UtilsTest.tempdir, name))
         other = cbfimage()
-        other.read(os.path.join(self.tempdir, name))
+        other.read(os.path.join(UtilsTest.tempdir, name))
         self.assertEqual(abs(obj.data - other.data).max(), 0, "data are the same")
         for key in obj.header:
-            if key in[ "filename", "X-Binary-Size-Padding"]:
+            if key in["filename", "X-Binary-Size-Padding"]:
                 continue
             self.assertTrue(key in other.header, "Key %s is in header" % key)
             self.assertEqual(obj.header[key], other.header[key], "value are the same for key %s" % key)
+        os.unlink(os.path.join(UtilsTest.tempdir, name))
 
     def test_byte_offset(self):
-        """ check byte offset algorythm"""
+        """ check byte offset algorithm"""
         cbf = fabio.open(self.cbf_filename)
-        starter = "\x0c\x1a\x04\xd5"
-        startPos = cbf.cif["_array_data.data"].find(starter) + 4
-        data = cbf.cif["_array_data.data"][ startPos: startPos + int(cbf.header["X-Binary-Size"])]
+        starter = b"\x0c\x1a\x04\xd5"
+        cbs = cbf.cbs
+        startPos = cbs.find(starter) + 4
+        data = cbs[startPos: startPos + int(cbf.header["X-Binary-Size"])]
         startTime = time.time()
-        numpyRes = decByteOffet_numpy(data, size=cbf.dim1 * cbf.dim2)
+        numpyRes = decByteOffset_numpy(data, size=cbf.dim1 * cbf.dim2)
         tNumpy = time.time() - startTime
         logger.info("Timing for Numpy method : %.3fs" % tNumpy)
 
-#        startTime = time.time()
-#        weaveRes = cbfimage.analyseWeave(data, size=cbf.dim1 * cbf.dim2)
-#        tWeave = time.time() - startTime
-#        delta = abs(numpyRes - weaveRes).max()
-#        self.assertAlmostEqual(0, delta)
-#        logger.info("Timing for Weave method : %.3fs, max delta=%s" % (tWeave, delta))
-#
-#        startTime = time.time()
-#        pythonRes = decByteOffet_numpy(data, size=cbf.dim1 * cbf.dim2)
-#        tPython = time.time() - startTime
-#        delta = abs(numpyRes - pythonRes).max()
-#        self.assertAlmostEqual(0, delta)
-#        logger.info("Timing for Python method : %.3fs, max delta= %s" % (tPython, delta))
-
         startTime = time.time()
-        cythonRes = decByteOffet_cython(stream=data, size=cbf.dim1 * cbf.dim2)
+        cythonRes = decByteOffset_cython(stream=data, size=cbf.dim1 * cbf.dim2)
         tCython = time.time() - startTime
         delta = abs(numpyRes - cythonRes).max()
         self.assertAlmostEqual(0, delta)
@@ -122,11 +122,11 @@ class test_cbfimage_reader(unittest.TestCase):
         name = os.path.basename(self.cbf_filename)
         obj = fabio.open(self.cbf_filename)
         new = fabio.cbfimage.cbfimage(data=obj.data, header=obj.header)
-        new.write(os.path.join(self.tempdir, name))
-        other = fabio.open(os.path.join(self.tempdir, name))
+        new.write(os.path.join(UtilsTest.tempdir, name))
+        other = fabio.open(os.path.join(UtilsTest.tempdir, name))
         self.assertEqual(abs(obj.data - other.data).max(), 0, "data are the same")
         for key in obj.header:
-            if key in[ "filename", "X-Binary-Size-Padding"]:
+            if key in["filename", "X-Binary-Size-Padding"]:
                 continue
             self.assertTrue(key in other.header, "Key %s is in header" % key)
             self.assertEqual(obj.header[key], other.header[key], "value are the same for key %s [%s|%s]" % (key, obj.header[key], other.header[key]))
@@ -138,11 +138,11 @@ class test_cbfimage_reader(unittest.TestCase):
         name = os.path.basename(self.cbf_filename)
         obj = fabio.open(self.cbf_filename)
         new = obj.convert("cbf")
-        new.write(os.path.join(self.tempdir, name))
-        other = fabio.open(os.path.join(self.tempdir, name))
+        new.write(os.path.join(UtilsTest.tempdir, name))
+        other = fabio.open(os.path.join(UtilsTest.tempdir, name))
         self.assertEqual(abs(obj.data - other.data).max(), 0, "data are the same")
         for key in obj.header:
-            if key in[ "filename", "X-Binary-Size-Padding"]:
+            if key in["filename", "X-Binary-Size-Padding"]:
                 continue
             self.assertTrue(key in other.header, "Key %s is in header" % key)
             self.assertEqual(obj.header[key], other.header[key], "value are the same for key %s [%s|%s]" % (key, obj.header[key], other.header[key]))
@@ -153,30 +153,28 @@ class test_cbfimage_reader(unittest.TestCase):
         """
         name = unicode(os.path.basename(self.cbf_filename))
         obj = fabio.open(self.cbf_filename)
-        obj.write(os.path.join(self.tempdir, name))
-        other = fabio.open(os.path.join(self.tempdir, name))
+        obj.write(os.path.join(UtilsTest.tempdir, name))
+        other = fabio.open(os.path.join(UtilsTest.tempdir, name))
         self.assertEqual(abs(obj.data - other.data).max(), 0, "data are the same")
         for key in obj.header:
-            if key in[ "filename", "X-Binary-Size-Padding"]:
+            if key in["filename", "X-Binary-Size-Padding"]:
                 continue
             self.assertTrue(key in other.header, "Key %s is in header" % key)
             self.assertEqual(obj.header[key], other.header[key], "value are the same for key %s [%s|%s]" % (key, obj.header[key], other.header[key]))
 
 
-def test_suite_all_cbf():
-    testSuite = unittest.TestSuite()
-    testSuite.addTest(test_cbfimage_reader("test_read"))
-    testSuite.addTest(test_cbfimage_reader("test_write"))
-    testSuite.addTest(test_cbfimage_reader("test_byte_offset"))
-    testSuite.addTest(test_cbfimage_reader("test_consitency_manual"))
-    testSuite.addTest(test_cbfimage_reader("test_consitency_convert"))
-    testSuite.addTest(test_cbfimage_reader("test_unicode"))
+def suite():
+    testsuite = unittest.TestSuite()
+    testsuite.addTest(TestCbfReader("test_read"))
+    testsuite.addTest(TestCbfReader("test_write"))
+    testsuite.addTest(TestCbfReader("test_byte_offset"))
+    testsuite.addTest(TestCbfReader("test_consitency_manual"))
+    testsuite.addTest(TestCbfReader("test_consitency_convert"))
+    testsuite.addTest(TestCbfReader("test_unicode"))
 
-    return testSuite
+    return testsuite
 
 if __name__ == '__main__':
-
-    mysuite = test_suite_all_cbf()
     runner = unittest.TextTestRunner()
-    runner.run(mysuite)
+    runner.run(suite())
 
