@@ -1,5 +1,26 @@
 #!/usr/bin/env python
-# -*- coding: utf8 -*- 
+# -*- coding: utf-8 -*-
+#
+#    Project: Fable Input Output
+#             https://github.com/kif/fabio
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 
 
 """
@@ -7,31 +28,21 @@ Test cases for filename deconstruction
 
 
 testsuite by Jerome Kieffer (Jerome.Kieffer@esrf.eu)
+28/11/2014
 """
+from __future__ import print_function, with_statement, division, absolute_import
+import unittest
+import sys
+import os
 
-import unittest, sys, os, logging
-logger = logging.getLogger("testfilenames")
-force_build = False
+if __name__ == '__main__':
+    import pkgutil
+    __path__ = pkgutil.extend_path([os.path.dirname(__file__)], "fabio.test")
+from .utilstest import UtilsTest
 
-for opts in sys.argv[:]:
-    if opts in ["-d", "--debug"]:
-        logging.basicConfig(level=logging.DEBUG)
-        sys.argv.pop(sys.argv.index(opts))
-    elif opts in ["-i", "--info"]:
-        logging.basicConfig(level=logging.INFO)
-        sys.argv.pop(sys.argv.index(opts))
-    elif opts in ["-f", "--force"]:
-        force_build = True
-        sys.argv.pop(sys.argv.index(opts))
-try:
-    logger.debug("Tests loaded from file: %s" % __file__)
-except:
-    __file__ = os.getcwd()
 
-from utilstest import UtilsTest
-if force_build:
-    UtilsTest.forceBuild()
-import fabio
+logger = UtilsTest.get_logger(__file__)
+fabio = sys.modules["fabio"]
 
 CASES = [
     (1, 'edf', "data0001.edf"),
@@ -52,8 +63,8 @@ CASES = [
     (99, 'bruker', "sucrose101.0099.gz"),
     (2, 'fit2dmask', "fit2d.msk"),
     (None, 'fit2dmask', "mymask.msk"),
-    (670005, 'edf' , 'S82P670005.edf'),
-    (670005, 'edf' , 'S82P670005.edf.gz'),
+    (670005, 'edf', 'S82P670005.edf'),
+    (670005, 'edf', 'S82P670005.edf.gz'),
     # based on only the name it can be either img or oxd
     (1     , 'adsc_or_OXD_or_HiPiC_or_raxis' , 'mb_LP_1_001.img'),
     (2     , 'adsc_or_OXD_or_HiPiC_or_raxis' , 'mb_LP_1_002.img.gz'),
@@ -80,18 +91,18 @@ MORE_CASES = [
     ]
 
 
-class testfilenames(unittest.TestCase):
+class TestFilenames(unittest.TestCase):
     """ check the name -> number, type conversions """
 
     def test_many_cases(self):
         """ loop over CASES """
         for num, typ, name in CASES:
             obj = fabio.FilenameObject(filename=name)
-            self.assertEqual(num, obj.num , name + " num=" + str(num) + \
+            self.assertEqual(num, obj.num, name + " num=" + str(num) + \
                                                  " != obj.num=" + str(obj.num))
             self.assertEqual(typ, "_or_".join(obj.format),
                                  name + " " + "_or_".join(obj.format))
-            self.assertEqual(name, obj.tostring() , name + " " + obj.tostring())
+            self.assertEqual(name, obj.tostring(), name + " " + obj.tostring())
 
     def test_more_cases(self):
         for nname, oname, num in MORE_CASES:
@@ -104,17 +115,15 @@ class testfilenames(unittest.TestCase):
             self.assertEqual(name, nname)
 
 
-def test_suite_all_filenames():
-    testSuite = unittest.TestSuite()
+def suite():
+    testsuite = unittest.TestSuite()
 
-    testSuite.addTest(testfilenames("test_many_cases"))
-    testSuite.addTest(testfilenames("test_more_cases"))
-    testSuite.addTest(testfilenames("test_more_cases_jump"))
+    testsuite.addTest(TestFilenames("test_many_cases"))
+    testsuite.addTest(TestFilenames("test_more_cases"))
+    testsuite.addTest(TestFilenames("test_more_cases_jump"))
 
-    return testSuite
+    return testsuite
 
 if __name__ == '__main__':
-
-    mysuite = test_suite_all_filenames()
     runner = unittest.TextTestRunner()
-    runner.run(mysuite)
+    runner.run(suite())
